#include "Arduino.h"
#include "IridiumGPP.h"
#include <Adafruit_MPU6050.h>
#include <Adafruit_Sensor.h>
#include <Wire.h>
#include <SparkFun_I2C_GPS_Arduino_Library.h>
#include <TinyGPS++.h> 

Adafruit_MPU6050 mpu; // MPU object
I2CGPS myI2CGPS; // I2C object for GPS
TinyGPSPlus gps; // GPS object

double getLat(){
  if (myI2CGPS.begin() == false)
  {
    Serial.println("Module failed to respond. Please check wiring.");
    while (1); // Freeze!
  }
 
 while (myI2CGPS.available()) //available() returns the number of new bytes available from the GPS module
  {
    gps.encode(myI2CGPS.read()); //Feed the GPS parser
  }

  if (gps.time.isUpdated()) //Check to see if new GPS info is available
  {
  if (gps.location.isValid())
  {
    return gps.location.lat();
  }
  else
  {
    return 99999;
  }
    }

}

double getLng(){
  if (myI2CGPS.begin() == false)
  {
    Serial.println("Module failed to respond. Please check wiring.");
    while (1); //Freeze!
  }
 
 while (myI2CGPS.available()) //available() returns the number of new bytes available from the GPS module
  {
    gps.encode(myI2CGPS.read()); //Feed the GPS parser
  }

  if (gps.time.isUpdated()) //Check to see if new GPS info is available
  {
 if (gps.location.isValid())
  {
    return gps.location.lng();
  }
  else
  {
    return 99999;
  }
    }

}

sensors_vec_t getAcc(){
if (!mpu.begin()) {
  Serial.println("Failed to find MPU6050 chip");
  while (1) {
    delay(10);
  }
}

sensors_event_t a, g, temp;
mpu.getEvent(&a, &g, &temp);
return a.acceleration;
}

sensors_vec_t getGyro (){
  if (!mpu.begin()) {
      Serial.println("Failed to find MPU6050 chip");
      while (1) {
        delay(10);
      }
    }
  
  sensors_event_t a, g, temp;
  mpu.getEvent(&a, &g, &temp);
  return g.gyro;
}

float getTemp(){
  if (!mpu.begin()) {
      Serial.println("Failed to find MPU6050 chip");
      while (1) {
        delay(10);
      }
    }
  
  sensors_event_t a, g, temp;
  mpu.getEvent(&a, &g, &temp);
  return temp.temperature-15.98;
}

 void beginSerial(){ // Begins serial connection between ESP32 and Edge
  Serial2.begin(19200, SERIAL_8N1, 16, 17); // Slave Baud Rate (19200), RX pin (Serial 2 is 16), TX pin (Serial 2 is 17)
  Serial.begin(9600); // Master Baud Rate
}

void debug(){ // Enables for printing the response of the transciever
  String echo = "";
  int counter = 0;
  while (counter < 20) {
    echo = Serial2.readString();
    if (echo != "") {
      Serial.println(echo); 
      counter += 1;
      if (echo.indexOf("OK") > 0){
        Serial.println ("Echo: " + String(counter) + " | Command Successful");
        Serial.println("________________________________________________________");
        break;
      }
    }
    else{
      counter++;
    }
  }
  if (counter == 20){
    Serial.println("Echo Failed");
  }
}

void atCommand(String command){ // Sends the AT command to transciever
  Serial2.println(command + "\r");
  debug(); 
}

void radioON() { // Turns on the radio
  atCommand("AT*R1");
}

void radioOFF() { // Turns off the radio
  atCommand("AT*R0");
}

void sendMSG(String message){ // Sends a message
  atCommand("AT+SBDWT=" + message);
  atCommand("AT+SBDIX");
}

void defaultConfig(){ // Sets transciever to default configuration
  atCommand("AT&K0"); // Disable Flow Control (RTS & CTS)
  atCommand("AT*R1"); // Enables the Radio
  atCommand("AT+SBDMTA=1"); // Enables the SBD Ring Configuration
}